/*
** Copyright 2017 Double Precision, Inc.
** See COPYING for distribution information.
*/
#ifndef x_w_standard_comboboxlayoutmanagerobj_h
#define x_w_standard_comboboxlayoutmanagerobj_h

#include <x/w/standard_comboboxlayoutmanagerfwd.H>
#include <x/w/custom_comboboxlayoutmanagerobj.H>
#include <x/w/text_paramfwd.H>

LIBCXXW_NAMESPACE_START

//! Standard combo-box layout manager object.

//! \see standard_comboboxlayoutmanager

class standard_comboboxlayoutmanagerObj
	: public custom_comboboxlayoutmanagerObj {

public:
	//! Alias
	typedef custom_comboboxlayoutmanagerObj superclass_t;

	class LIBCXX_HIDDEN implObj;

	//! Internal implementation object.
	const ref<implObj> impl;

	//! Constructor
	standard_comboboxlayoutmanagerObj(const ref<implObj> &impl,
					  const ref<listlayoutmanagerObj
					  ::implObj>
					  &list_layout_impl) LIBCXX_HIDDEN;

	//! Destructor
	~standard_comboboxlayoutmanagerObj() LIBCXX_HIDDEN;

	//! Append a new item to the combo-box.

	void append_items(const std::vector<list_item_param> &items) override;

	//! Append a new item to the combo-box.

	void append_items(ONLY IN_THREAD,
			  const std::vector<list_item_param> &items) override;

	//! Insert a new item to the combo-box before an existing item.

	void insert_items(size_t i, const std::vector<list_item_param> &items)
		override;

	//! Insert a new item to the combo-box before an existing item.

	void insert_items(ONLY IN_THREAD,
			  size_t i, const std::vector<list_item_param> &items)
		override;

	//! Remove an item from the combo-box.
	void remove_item(ONLY IN_THREAD, size_t i) override;

	using custom_comboboxlayoutmanagerObj::remove_item;

	//! Replace an item.

	void replace_items(size_t item_number,
			  const std::vector<list_item_param> &items) override;

	//! Replace an item.

	void replace_items(ONLY IN_THREAD,
			   size_t item_number,
			   const std::vector<list_item_param> &items) override;

	//! Override replace_all().

	void replace_all_items(const std::vector<list_item_param> &items)
		override;

	//! Override replace_all().

	void replace_all_items(ONLY IN_THREAD,
			       const std::vector<list_item_param> &items)
		override;

	using superclass_t::resort_items;

	//! Override resort_items().
	void resort_items(ONLY IN_THREAD,
			  const std::vector<size_t> &indexes) override;

	//! Return combo-box item.
	text_param item(size_t i) const;

	//! Install a new selection changed callback.
	void selection_changed(const standard_combobox_selection_changed_t &);

};

LIBCXXW_NAMESPACE_END

#endif
